#!/bin/bash

##############################################################################
#
# Generic Version Comparison Tools
#
# !!! WARNING !!!
# the function f_abort must be defined by any module that sources
# this module.
#
##############################################################################

# set unknown versions to a format that can be parsed just in case
UNKNOWN_VERSION="0.0.0.0"

#-----------------------------------------------------------------------------
# f_getCurVersion()
#
# Determine the Current SANiQ Version
#-----------------------------------------------------------------------------
function f_getCurVersion {
	# determine if the current version information exists
	# it's possible that the filesystem with this file is not mounted
	if [ -e /etc/lefthand/system/.version ]; then
		CUR_SANIQ_VERSION=`cat /etc/lefthand/system/.version | head -1`
	else
		CUR_SANIQ_VERSION=${UNKNOWN_VERSION}
	fi

	# break the version number into different variables
	CUR_SANIQ_VERSION_MAJOR=`echo ${CUR_SANIQ_VERSION} | awk -F. '{print $1}'`
	CUR_SANIQ_VERSION_MINOR=`echo ${CUR_SANIQ_VERSION} | awk -F. '{print $2}'`
	CUR_SANIQ_VERSION_MICRO=`echo ${CUR_SANIQ_VERSION} | awk -F. '{print $3}'`
	CUR_SANIQ_VERSION_BUILD=`echo ${CUR_SANIQ_VERSION} | awk -F. '{print $4}'`

	# make variables available outside of the function
	export CUR_SANIQ_VERSION
	export CUR_SANIQ_VERSION_MAJOR
	export CUR_SANIQ_VERSION_MINOR
	export CUR_SANIQ_VERSION_MICRO
	export CUR_SANIQ_VERSION_BUILD
}


#-----------------------------------------------------------------------------
# f_compareVersions()
#
# Compare Two Versions
#
# WARNING: It is VERY important that this function does not produce any
#          output.  If it does it will prevent inline return status
#          checks to fail to operate as expected.  This is the only way
#          to make this function usable inside of a if statement since
#          bash does not have implicit return codes.
#
# Args:
#		VER1 - version number 1
#		VER2 - version number 2
#
# Returns:
#		0 - when VER1 < VER2
#		1 - when VER1 and VER2 are the same 
#		2 - when VER1 > VER2
#-----------------------------------------------------------------------------
function f_compareVersions {
	if [ $# -ne 2 ]; then
		echo "ERROR: f_compareVersions wrong number of arguments"
		f_abort ${EXIT_STATUS_FUNCTION_ERROR}
	fi

	VER1=$1
	VER1_MAJOR=`echo ${VER1} | awk -F. '{print $1}'`
	VER1_MINOR=`echo ${VER1} | awk -F. '{print $2}'`
	VER1_MICRO=`echo ${VER1} | awk -F. '{print $3}'`
	VER1_BUILD=`echo ${VER1} | awk -F. '{print $4}'`

	VER2=$2
	VER2_MAJOR=`echo ${VER2} | awk -F. '{print $1}'`
	VER2_MINOR=`echo ${VER2} | awk -F. '{print $2}'`
	VER2_MICRO=`echo ${VER2} | awk -F. '{print $3}'`
	VER2_BUILD=`echo ${VER2} | awk -F. '{print $4}'`

	# ensure that each version has the same details
	# This allows A.B.C to be compared to X.Y.Z.B
	# Specifically allow 6.6.0 to be comapred to 6.6.00.0099
	#     and considred to be equal.
	if [ "${VER1_MAJOR}" == "" -o "${VER2_MAJOR}" == "" ]; then
		VER1_MAJOR="0"
		VER2_MAJOR="0"
	fi

	if [ "${VER1_MINOR}" == "" -o "${VER2_MINOR}" == "" ]; then
		VER1_MINOR="0"
		VER2_MINOR="0"
	fi

	if [ "${VER1_MICRO}" == "" -o "${VER2_MICRO}" == "" ]; then
		VER1_MICRO="0"
		VER2_MICRO="0"
	fi

	if [ "${VER1_BUILD}" == "" -o "${VER2_BUILD}" == "" ]; then
		VER1_BUILD="0"
		VER2_BUILD="0"
	fi

	# compare the versions
	if [ "${VER1_MAJOR}" -lt "${VER2_MAJOR}" ]; then
		return 0
	elif [ "${VER1_MAJOR}" -gt "${VER2_MAJOR}" ]; then
		return 2
	else
		if [ "${VER1_MINOR}" -lt "${VER2_MINOR}" ]; then
			return 0
		elif [ "${VER1_MINOR}" -gt "${VER2_MINOR}" ]; then
			return 2
		else
			if [ "${VER1_MICRO}" -lt "${VER2_MICRO}" ]; then
				return 0
			elif [ "${VER1_MICRO}" -gt "${VER2_MICRO}" ]; then
				return 2
			else
				if [ "${VER1_BUILD}" -lt "${VER2_BUILD}" ]; then
					return 0
				elif [ "${VER1_BUILD}" -gt "${VER2_BUILD}" ]; then
					return 2
				else
					return 1
				fi
			fi
		fi
	fi
}

#-----------------------------------------------------------------------------
# f_isVersionEqual()
#
# WARNING: It is VERY important that this function does not produce any
#          output.  If it does it will prevent inline return status
#          checks to fail to operate as expected.  This is the only way
#          to make this function usable inside of a if statement since
#          bash does not have implicit return codes.
#
# Is $1 equal to $2?
#-----------------------------------------------------------------------------
function f_isVersionEqual {
	if [ $# -ne 2 ]; then
		echo "ERROR: f_compareVersions wrong number of arguments"
		f_abort ${EXIT_STATUS_FUNCTION_ERROR}
	fi

	f_compareVersions $1 $2
	result=$?
	if [ ${result} -eq 1 ]; then
		return ${TRUE}
	else
		return ${FALSE}
	fi
}

#-----------------------------------------------------------------------------
# f_isVersionLessThan()
#
# WARNING: It is VERY important that this function does not produce any
#          output.  If it does it will prevent inline return status
#          checks to fail to operate as expected.  This is the only way
#          to make this function usable inside of a if statement since
#          bash does not have implicit return codes.
#
# Is $1 less than $2?
#-----------------------------------------------------------------------------
function f_isVersionLessThan {
	if [ $# -ne 2 ]; then
		echo "ERROR: f_compareVersions wrong number of arguments"
		f_abort ${EXIT_STATUS_FUNCTION_ERROR}
	fi

	f_compareVersions $1 $2
	result=$?
	if [ ${result} -eq 0 ]; then
		return ${TRUE}
	else
		return ${FALSE}
	fi
}

#-----------------------------------------------------------------------------
# f_isVersionLessThanOrEqual()
#
# WARNING: It is VERY important that this function does not produce any
#          output.  If it does it will prevent inline return status
#          checks to fail to operate as expected.  This is the only way
#          to make this function usable inside of a if statement since
#          bash does not have implicit return codes.
#
# Is $1 less than or equal to $2?
#-----------------------------------------------------------------------------
function f_isVersionLessThanOrEqual {
	if [ $# -ne 2 ]; then
		echo "ERROR: f_compareVersions wrong number of arguments"
		f_abort ${EXIT_STATUS_FUNCTION_ERROR}
	fi

	f_compareVersions $1 $2
	result=$?
	if [ ${result} -le 1 ]; then
		return ${TRUE}
	else
		return ${FALSE}
	fi
}

#-----------------------------------------------------------------------------
# f_isVersionGreaterThanOrEqual()
#
# WARNING: It is VERY important that this function does not produce any
#          output.  If it does it will prevent inline return status
#          checks to fail to operate as expected.  This is the only way
#          to make this function usable inside of a if statement since
#          bash does not have implicit return codes.
#
# Is $1 greater than or equal to $2?
#-----------------------------------------------------------------------------
function f_isVersionGreaterThanOrEqual {
	if [ $# -ne 2 ]; then
		echo "ERROR: f_compareVersions wrong number of arguments"
		f_abort ${EXIT_STATUS_FUNCTION_ERROR}
	fi

	f_compareVersions $1 $2
	result=$?
	if [ ${result} -ge 1 ]; then
		return ${TRUE}
	else
		return ${FALSE}
	fi
}

#-----------------------------------------------------------------------------
# f_isVersionGreaterThan()
#
# WARNING: It is VERY important that this function does not produce any
#          output.  If it does it will prevent inline return status
#          checks to fail to operate as expected.  This is the only way
#          to make this function usable inside of a if statement since
#          bash does not have implicit return codes.
#
# Is $1 greater than $2?
#-----------------------------------------------------------------------------
function f_isVersionGreaterThan {
	if [ $# -ne 2 ]; then
		echo "ERROR: f_compareVersions wrong number of arguments"
		f_abort ${EXIT_STATUS_FUNCTION_ERROR}
	fi

	f_compareVersions $1 $2
	result=$?
	if [ ${result} -eq 2 ]; then
		return ${TRUE}
	else
		return ${FALSE}
	fi
}

