#!/bin/bash

#------------------------------------------------------------------------------
# Description
# This patch delivers the 05/05/2011 System ROM (BIOS) update for warrior
#------------------------------------------------------------------------------
DEBUG=

install_log=$1
modelName=`hwid --name`

# BASE_DIR is determined in apply_patch.sh which calls this script
PATCH_BASE_DIR=$BASE_DIR

PATCH="10112-00"
LHN_SHUTDOWN="/etc/lefthand/brand/common/lhn_shutdown"

set -u # exit on unset variable
# [ "$DEBUG" ] && set -x # log commands that are executed

function print_debug {
	[ "$DEBUG" ] && print "DEBUG:" $*
}

# This function provides standard handling for cases where something goes wrong
# that only support will be able to recover.
function panic {
	ec=$1
	msg=$2
	print "$2"
	print "Installation of the patch was aborted due to error $ec. (`date`)"
	print "Call Customer Support."
	logger -p syslog.error -t "patch" "ERROR: $ec: Installation of patch ${PATCH_NUM} was aborted: $msg"
	exit $ec
}
function f_abort {
	panic $1 "f_abort"
}

function installModule {
	directory=$2
	filename=$1

	print_debug "installing $filename to $directory"

	cp -pf ${PATCH_BASE_DIR}/data/$filename $directory/$filename.${PATCH}
	retVal=$?
	if [ $retVal -ne 0 ] ; then
		panic 1 "Cannot copy new $filename to $directory.  Patch failed."
	fi

	# The module has been successfully copied, so now move it to it's
	# final resting place.  This should be atomic so we use mv.
	print_debug "replacing old $filename with new $filename"
	mv -f $directory/$filename.${PATCH} $directory/$filename
	retVal=$?
	if [ $retVal -ne 0 ] ; then
		panic 1 "Cannot rename $filename.${PATCH} to $filename.  Patch failed."
	fi
}

#-----------------------------------------------------------------------------
# f_disconnectCMC()
# Comes from swd-lhn-upgrade/trunk/common/bin/services.sh
#-----------------------------------------------------------------------------
 function f_disconnectCMC {
    export PROCESS_NAME="kaffe-bin"
    export PROCESS_PID=`pidof ${PROCESS_NAME}`
    echo "Attempting to Remove the CMC socket connection"
    echo "Process: `ps -p ${PROCESS_PID}`"
    kill -TERM ${PROCESS_PID}
    retVal=$?
    echo "Process: `ps -p ${PROCESS_PID}`"
    return ${retVal}
}

#-----------------------------------------------------------------------------
# f_resetIOStream()
# stop using file descriptors 4 and 5
# Comes from swd-lhn-upgrade/trunk/common/bin/logs.sh
#-----------------------------------------------------------------------------
function f_resetIOStream {
    # take a short break to make sure the last output is processed
    sleep 5
    echo "Reset filedescriptors"

    exec 4>&-
    exec 5>&-
}

# definitions needed for genVersion.sh
declare -rx TRUE=1
declare -rx FALSE=0

# genVersion.sh needs f_abort defined
source ${BASE_DIR}/genVersion.sh

#------------------------------------------------------------------------------
# Enforce version: only for 9.0 or greater
#------------------------------------------------------------------------------
f_getCurVersion
`f_isVersionGreaterThanOrEqual ${CUR_SANIQ_VERSION} "9"`
vrc=$?
print "versioning return code [$vrc]"
if [ $vrc -eq ${TRUE} ]; then
    echo "Current SANiQ Version: ${CUR_SANIQ_VERSION}"
else
    print "ERROR: This patch is NOT for SAN/IQ version ${CUR_SANIQ_VERSION}, patch failed!"
    declare -rx EXIT_STATUS_PATCH_FAILED_VERSION=20
    f_abort ${EXIT_STATUS_PATCH_FAILED_VERSION}
fi

#------------------------------------------------------------------------------
# Prechecks
#------------------------------------------------------------------------------


#------------------------------------------------------------------------------
# Stage 1
#------------------------------------------------------------------------------
dmidecode | grep 460 | grep G7
G7_mobo=$?
dmidecode | grep 460 | grep G6
G6_mobo=$?

if [ $G6_mobo -eq 0 ] ; then
    # This is a G6 motherboard
    installModule "CP015479.scexe" "/tmp"
    installModule "CP015479.scexe.md5" "/tmp"
    BIOS_Installer=/tmp/CP015479.scexe
    print "BIOS installer for bl460C G6 motherboard is $BIOS_Installer"
elif [ $G7_mobo -eq 0 ] ; then
    # This is a G7 motherboard
    installModule "CP015510.scexe" "/tmp"
    installModule "CP015510.scexe.md5" "/tmp"
    BIOS_Installer=/tmp/CP015510.scexe
    print "BIOS installer for bl460C G7 motherboard is $BIOS_Installer"
fi

# Check md5 sum so we're confident the .scexe file hasn't been corrupted during transmission and unpacking
pushd /tmp
md5sum --check ${BIOS_Installer}.md5
retval=$?
popd
if [ $retval -eq 0 ] ; then
    print "$BIOS_Installer matches ${BIOS_Installer}.md5, continue"
else
    print "$BIOS_Installer does not match ${BIOS_Installer}.md5, patch has been corrupted"
    declare -rx EXIT_STATUS_PATCH_FAILED_MD5=30
    f_abort ${EXIT_STATUS_PATCH_FAILED_MD5}
fi

print "Finished copying SmartComponents file [$BIOS_Installer]."
chmod 755 $BIOS_Installer

get_rev=get_bios_version.py
full_get_rev=/tmp/$get_rev
installModule "$get_rev" "/tmp"
print "Finished copying $get_rev"

#------------------------------------------------------------------------------
# Stage 1 complete, wait for stage-2 commit from CMC
#------------------------------------------------------------------------------
print "STATE: Stage 1 complete, waiting for commit from CMC ..."
print "INSTALL_STATUS_MESSAGE: done [0]"

timeout=0
let HRS24=86400
let "WEEK=${HRS24}*7"

while [ ! -e /tmp/Stage2File.tmp ]
do
	sleep 2
	let timeout+=2
	if [ $timeout -ge ${WEEK} ] ; then
		print "ERROR: Timed out $timeout seconds waiting for stage 2 commit from CMC"
		exit 207
	fi
done
print "STATE: Commit received from CMC, beginning stage-2 ..."

#------------------------------------------------------------------------------
# Apply/make changes
#------------------------------------------------------------------------------
# Prepare to replace modules, shared objects and/or NIC firmware.
pushd /root > /dev/null

print "Check System ROM (BIOS) version ..."
install_bios=1

python $full_get_rev
bios_rc=$?

if [ $bios_rc -eq 4 ]; then
    print "Current BIOS more recent, not really installing new BIOS"
    install_bios=0
elif [ $bios_rc -eq -1 ]; then
    print "ERROR: Problem running the [${full_get_rev}] script."
    exit $bios_rc
fi

if [ ${install_bios} -eq 1 ]; then
    print "Begin installing new software modules"
    
    # /tmp/norun allows us to test the patch w/out having to install
    # the SmartComponent.
    if [ -e /tmp/norun ]; then
        echo "Found /tmp/norun, not running SmartComponent."
        sc_rc=0
    else
        # We must stop hydra before installing f/w! Otherwise the CMC may re-discover this node
        # while f/w is being updated (while hydra is up) and think the node has already rebooted.
        servicectl hydra stop

        echo "yes" | $BIOS_Installer -s
        sc_rc=$?
    fi

    print "Smart Component return code [$sc_rc]"
    print "Finished installing new software modules"

    if [ $sc_rc -ne 0 ] ; then
        print "Error [$sc_rc] encountered while installing patch."
        exit 1
    fi
fi

popd > /dev/null

#------------------------------------------------------------------------------
# Postchecks
#------------------------------------------------------------------------------


#------------------------------------------------------------------------------
# done
#------------------------------------------------------------------------------
print "Patch ${PATCH_NUM} has been successfully installed."


#------------------------------------------------------------------------------
# Only wite the patch.xml file if your patch has been successful.
#------------------------------------------------------------------------------
# Success: install the patch information file
cp -f ${PATCH_BASE_DIR}/${XMLFILE} ${SYSTEM_PATCH_DIR}
if [ $? -ne 0 ] ; then
	panic 6 "Unable to install the patch information file (${XMLFILE})"
else
	print "Wrote ${PATCH_BASE_DIR}/${XMLFILE} to ${SYSTEM_PATCH_DIR}"
fi

#------------------------------------------------------------------------------
# REBOOT
#------------------------------------------------------------------------------
print "Patch ${PATCH_NUM} has been successfully installed."
print "Completed installation of patch ${PATCH_NUM}. (`date`)"

# THe f_disconnectCMC & f_resetIOStream functions actually come from the
# upgrade backend, upgrade/common/bin/service.sh & upgrade/common/bin/logs.sh
# respectively.
# This is how 2-stage online upgrades disconnect from the CMC and close the
# output streams created to talk to the upgrade server.
f_disconnectCMC
f_resetIOStream

# reboot
if [ ${install_bios} -eq 1 ]; then
    bash -c "${LHN_SHUTDOWN} -r now &"
fi

exit 0
